// Generated by CoffeeScript 1.12.6

/*
Copyright 2016-2017 Resin.io

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */
var Promise;

Promise = require('bluebird');

exports.getGroupDefaults = function(group) {
  var _;
  _ = require('lodash');
  return _.chain(group).get('options').map(function(question) {
    return [question.name, question["default"]];
  }).object().value();
};

exports.stateToString = function(state) {
  var _str, chalk, percentage, result;
  _str = require('underscore.string');
  chalk = require('chalk');
  percentage = _str.lpad(state.percentage, 3, '0') + '%';
  result = (chalk.blue(percentage)) + " " + (chalk.cyan(state.operation.command));
  switch (state.operation.command) {
    case 'copy':
      return result + " " + state.operation.from.path + " -> " + state.operation.to.path;
    case 'replace':
      return result + " " + state.operation.file.path + ", " + state.operation.copy + " -> " + state.operation.replace;
    case 'run-script':
      return result + " " + state.operation.script;
    default:
      throw new Error("Unsupported operation: " + state.operation.type);
  }
};

exports.sudo = function(command) {
  var _, os, presidentExecuteAsync;
  _ = require('lodash');
  os = require('os');
  if (os.platform() !== 'win32') {
    console.log('If asked please type your computer password to continue');
  }
  command = _.union(_.take(process.argv, 2), command);
  presidentExecuteAsync = Promise.promisify(require('president').execute);
  return presidentExecuteAsync(command);
};

exports.getManifest = function(image, deviceType) {
  var imagefs, resin, rindle;
  rindle = require('rindle');
  imagefs = require('resin-image-fs');
  resin = require('resin-sdk-preconfigured');
  return imagefs.read({
    image: image,
    partition: {
      primary: 1
    },
    path: '/device-type.json'
  }).then(rindle.extractAsync).then(JSON.parse)["catch"](function() {
    return resin.models.device.getManifestBySlug(deviceType);
  });
};

exports.osProgressHandler = function(step) {
  var progressBars, rindle, visuals;
  rindle = require('rindle');
  visuals = require('resin-cli-visuals');
  step.on('stdout', process.stdout.write.bind(process.stdout));
  step.on('stderr', process.stderr.write.bind(process.stderr));
  step.on('state', function(state) {
    if (state.operation.command === 'burn') {
      return;
    }
    return console.log(exports.stateToString(state));
  });
  progressBars = {
    write: new visuals.Progress('Writing Device OS'),
    check: new visuals.Progress('Validating Device OS')
  };
  step.on('burn', function(state) {
    return progressBars[state.type].update(state);
  });
  return rindle.wait(step);
};

exports.getAppInfo = function(application) {
  var _, resin;
  resin = require('resin-sdk-preconfigured');
  _ = require('lodash');
  return Promise.join(resin.models.application.get(application), resin.models.config.getDeviceTypes(), function(app, config) {
    config = _.find(config, {
      'slug': app.device_type
    });
    if (config == null) {
      throw new Error('Could not read application information!');
    }
    app.arch = config.arch;
    return app;
  });
};

exports.getSubShellCommand = function(command) {
  var os;
  os = require('os');
  if (os.platform() === 'win32') {
    return {
      program: 'cmd.exe',
      args: ['/s', '/c', command]
    };
  } else {
    return {
      program: '/bin/sh',
      args: ['-c', command]
    };
  }
};
